package com.hgzx.plp.ds.oc;

import java.io.File;
import java.io.InputStream;

import org.apache.catalina.connector.Connector;
import org.apache.commons.io.FileUtils;
import org.apache.coyote.http11.Http11NioProtocol;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.web.embedded.tomcat.TomcatServletWebServerFactory;
import org.springframework.context.annotation.Bean;

@SpringBootApplication
public class DsOcApp {
	
	public static void main(String[] args) {
		SpringApplication.run(DsOcApp.class, args);
	}
	
	@Value("${https.port}")
    private Integer port;

    @Value("${https.ssl.key-store-password}")
    private String keyStorePassword;

    @Value("${https.ssl.key-password}")
    private String keyPassword;
    
    @Value("${https.ssl.key-store}")
    private String keyStore;

	@Bean
    public TomcatServletWebServerFactory servletContainer() {
		TomcatServletWebServerFactory tomcat = new TomcatServletWebServerFactory();
        tomcat.addAdditionalTomcatConnectors(createSslConnector()); // 添加http
        return tomcat;
    }

	private Connector createSslConnector() {
        Connector connector = new Connector("org.apache.coyote.http11.Http11NioProtocol");
        Http11NioProtocol protocol = (Http11NioProtocol) connector.getProtocolHandler();
	    try {
	    	InputStream inputStream = this.getClass().getResourceAsStream("/" + keyStore);
	    	File targetFile = new File(System.getProperty("java.io.tmpdir") + keyStore);
	    	FileUtils.copyInputStreamToFile(inputStream, targetFile);
	    	connector.setScheme("https");
	        connector.setSecure(true);
	        connector.setPort(port);
	        protocol.setSSLEnabled(true);
	        protocol.setKeystoreFile(targetFile.getAbsolutePath());
	        protocol.setKeystorePass(keyStorePassword);
	        protocol.setKeyPass(keyPassword);
	        return connector;
		} catch (Exception ex) {
	        throw new IllegalStateException("can't access keystore: [" + "keystore" + "] or truststore: [" + "keystore" + "]", ex);
	    }
    }
}
