package com.hgzx.plp.ds.oc.common.utils;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.security.KeyStore;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HttpsUtil {
	
	private static final Logger logger = LoggerFactory.getLogger(HttpsUtil.class);
	
	private final static String CONTENT_LENGTH = "Content-Length";

	private CloseableHttpClient httpClient;
	
	/**	
	 * Two-Way Authentication In the two-way authentication, the client needs: 
	 * 1 Import your own certificate for server verification; 
	 * 2 Import the CA certificate of the server, and use the CA certificate to verify the certificate sent by the server; 
	 * 3 Set the domain name to not verify (Non-commercial IoT platform, no use domain name access.)
	 * */
	public void createSSLClient(String certPath, String certPwd, String caPath, String caPwd) throws Exception {
		
		KeyStore selfCert = KeyStore.getInstance("pkcs12");
		selfCert.load(this.getClass().getResourceAsStream("/" + certPath), certPwd.toCharArray());
		KeyManagerFactory kmf = KeyManagerFactory.getInstance("sunx509");
		kmf.init(selfCert, certPwd.toCharArray());

		// 2 Import the CA certificate of the server,
		KeyStore caCert = KeyStore.getInstance("jks");
		
		caCert.load(this.getClass().getResourceAsStream("/" + caPath), caPwd.toCharArray());
		TrustManagerFactory tmf = TrustManagerFactory.getInstance("sunx509");
		tmf.init(caCert);
		
		SSLContext sc = SSLContext.getInstance("TLS");
		sc.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);

		HostnameVerifier hostNameVerifier = NoopHostnameVerifier.INSTANCE;
		SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sc, hostNameVerifier);
		httpClient = HttpClients.custom().setSSLSocketFactory(sslsf).build();
	}

	public HttpResponse doPostJson(String url, Map<String, String> headerMap, String content) {
		HttpPost request = new HttpPost(url);
		addRequestHeader(request, headerMap);
		request.setEntity(new StringEntity(content, ContentType.APPLICATION_JSON));
		return executeHttpRequest(request);
	}
	
    public StreamClosedHttpResponse doPostMultipartFile(String url, Map<String, String> headerMap, File file) {
        HttpPost request = new HttpPost(url);
        addRequestHeader(request, headerMap);
        
        FileBody fileBody = new FileBody(file);
        // Content-Type:multipart/form-data; boundary=----WebKitFormBoundarypJTQXMOZ3dLEzJ4b
        HttpEntity reqEntity = (HttpEntity) MultipartEntityBuilder.create().addPart("file", fileBody).build();
        request.setEntity(reqEntity);
        
        return (StreamClosedHttpResponse) executeHttpRequest(request);
    }

	public StreamClosedHttpResponse doPostJsonGetStatusLine(String url, Map<String, String> headerMap, String content) {
		HttpPost request = new HttpPost(url);
		addRequestHeader(request, headerMap);
		request.setEntity(new StringEntity(content, ContentType.APPLICATION_JSON));
		HttpResponse response = executeHttpRequest(request);
		if (null == response) {
			logger.debug("The response body is null.");
		}
		return (StreamClosedHttpResponse) response;
	}

	public StreamClosedHttpResponse doPostJsonGetStatusLine(String url, String content) {
		HttpPost request = new HttpPost(url);
		request.setEntity(new StringEntity(content, ContentType.APPLICATION_JSON));
		HttpResponse response = executeHttpRequest(request);
		if (null == response) {
			logger.debug("The response body is null.");
		}
		return (StreamClosedHttpResponse) response;
	}
	
	private List<NameValuePair> paramsConverter(Map<String, String> params) {
		List<NameValuePair> nvps = new LinkedList<NameValuePair>();
		Set<Map.Entry<String, String>> paramsSet = params.entrySet();
		for (Map.Entry<String, String> paramEntry : paramsSet) {
			nvps.add(new BasicNameValuePair(paramEntry.getKey(), paramEntry.getValue()));
		}
		return nvps;
	}

	public StreamClosedHttpResponse doPostFormUrlEncodedGetStatusLine(String url, Map<String, String> formParams) throws Exception {
		HttpPost request = new HttpPost(url);
		request.setEntity(new UrlEncodedFormEntity(paramsConverter(formParams)));
		HttpResponse response = executeHttpRequest(request);
		if (null == response) {
			logger.error("The response body is null.");
			throw new Exception();
		}
		return (StreamClosedHttpResponse) response;
	}

	public HttpResponse doPutJson(String url, Map<String, String> headerMap, String content) {
		HttpPut request = new HttpPut(url);
		addRequestHeader(request, headerMap);
		request.setEntity(new StringEntity(content, ContentType.APPLICATION_JSON));
		return executeHttpRequest(request);
	}
    
	public HttpResponse doPut(String url, Map<String, String> headerMap) {
		HttpPut request = new HttpPut(url);
		addRequestHeader(request, headerMap);
		return executeHttpRequest(request);
	}
	
	public StreamClosedHttpResponse doPutJsonGetStatusLine(String url, Map<String, String> headerMap, String content) {
		HttpResponse response = doPutJson(url, headerMap, content);
		if (null == response) {
			logger.error("The response body is null.");
		}
		return (StreamClosedHttpResponse) response;
	}
	
	public StreamClosedHttpResponse doPutGetStatusLine(String url, Map<String, String> headerMap) {
		HttpResponse response = doPut(url, headerMap);
		if (null == response) {
			logger.error("The response body is null.");
		}
		return (StreamClosedHttpResponse) response;
	}

	public HttpResponse doGetWithParas(String url, Map<String, String> queryParams, Map<String, String> headerMap) throws Exception {
		HttpGet request = new HttpGet();
		addRequestHeader(request, headerMap);
		URIBuilder builder;
		try {
			builder = new URIBuilder(url);
		} catch (URISyntaxException e) {
			logger.error("URISyntaxException: {}", e);
			throw new Exception(e);
		}

		if (queryParams != null && !queryParams.isEmpty()) {
			builder.setParameters(paramsConverter(queryParams));
		}
		request.setURI(builder.build());

		return executeHttpRequest(request);
	}

	public StreamClosedHttpResponse doGetWithParasGetStatusLine(String url, Map<String, String> queryParams, Map<String, String> headerMap) throws Exception {
		HttpResponse response = doGetWithParas(url, queryParams, headerMap);
		if (null == response) {
			logger.error("The response body is null.");
		}
		return (StreamClosedHttpResponse) response;
	}

	public HttpResponse doDelete(String url, Map<String, String> headerMap) {
		HttpDelete request = new HttpDelete(url);
		addRequestHeader(request, headerMap);
		return executeHttpRequest(request);
	}

	public StreamClosedHttpResponse doDeleteGetStatusLine(String url, Map<String, String> headerMap) {
		HttpResponse response = doDelete(url, headerMap);
		if (null == response) {
			logger.error("The response body is null.");
		}
		return (StreamClosedHttpResponse) response;
	}

	private static void addRequestHeader(HttpUriRequest request, Map<String, String> headerMap) {
		if (headerMap == null) {
			return;
		}

		for (String headerName : headerMap.keySet()) {
			if (CONTENT_LENGTH.equalsIgnoreCase(headerName)) {
				continue;
			}

			String headerValue = headerMap.get(headerName);
			request.addHeader(headerName, headerValue);
		}
	}

	private HttpResponse executeHttpRequest(HttpUriRequest request) {
		HttpResponse response = null;
		try {
			response = httpClient.execute(request);
		} catch (Exception e) {
			logger.error("executeHttpRequest failed. error: {}", e);
		} finally {
			try {
				response = new StreamClosedHttpResponse(response);
				if(request != null) {
					httpClient.close();
					logger.debug("close http request.");
				}
			} catch (IOException e) {
				logger.error("IOException. error Message: {}", e.getMessage());
			}
		}
		return response;
	}

	public String getHttpResponseBody(HttpResponse response) throws UnsupportedOperationException, IOException {
		if (response == null) {
			return null;
		}
		String body = null;
		if (response instanceof StreamClosedHttpResponse) {
			body = ((StreamClosedHttpResponse) response).getContent();
		} else {
			HttpEntity entity = response.getEntity();
			if (entity != null && entity.isStreaming()) {
				String encoding = entity.getContentEncoding() != null ? entity.getContentEncoding().getValue() : null;
				body = StreamUtil.inputStream2String(entity.getContent(), encoding);
			}
		}
		return body;
	}
}
