/*
 * File Name: com.huawei.utils.Constant.java
 *
 * Copyright Notice:
 *      Copyright  1998-2008, Huawei Technologies Co., Ltd.  ALL Rights Reserved.
 *
 *      Warning: This computer software sourcecode is protected by copyright law
 *      and international treaties. Unauthorized reproduction or distribution
 *      of this sourcecode, or any portion of it, may result in severe civil and
 *      criminal penalties, and will be prosecuted to the maximum extent
 *      possible under the law.
 */
package com.hgzx.plp.ds.oc.config;

import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

@Configuration
@ConfigurationProperties(prefix = "ocds.na")
public class OcConfig {

	/**
	 * oc平台对接地址
	 */
	private String baseUrl;

	/**
	 * 应用ID
	 */
	private String appId;

	/**
	 * 应用密码
	 */
	private String secret;

	/**
	 * 回调地址
	 */
	private String callbackBaseUrl;
	
	/**
	 * 设备添加回调
	 */
	private String deviceAddedCallbackUrl;
	
	/**
	 * 设备信息变化回调
	 */
	private String deviceInfoChangedCallbackUrl;
	
	/**
	 * 设备数据变化回调
	 */
	private String deviceDataChangedCallbackUrl;
	
	/**
	 * 设备删除变化回调
	 */
	private String deviceDeletedCallbackUrl;
	
	/**
	 * 消息确认回调
	 */
	private String messageConfirmCallbackUrl;
	
	/**
	 * 服务变化回调
	 */
	private String serviceInfoChangedCallbackUrl;
	
	/**
	 * 命令响应回调
	 */
	private String commandRspCallbackUrl;
	
	/**
	 * 设备事件变化回调
	 */
	private String deviceEventCallbackUrl;
	
	/**
	 * 规则事件变化回调
	 */
	private String ruleEventCallbackUrl;

	/**
	 * 设备数据变化回调
	 */
	private String deviceDatasChangedCallbackUrl;

	/**
	 * 命令执行回调地址
	 */
	private String reportCmdExecResultCallbackUrl;

	// Paths of certificates.
	private String selfCertPath;
	private String trustCaPath;

	// Password of certificates.
	public String selfCertPwd;
	public String trustCaPwd;

	// ************* The following constants do not need to be modified *************//

	/*
	 * request header 
	 * 1. HEADER_APP_KEY 
	 * 2. HEADER_APP_AUTH
	 */
	public static final String HEADER_APP_KEY = "app_key";
	public static final String HEADER_APP_AUTH = "Authorization";

	/*
	 * Application Access Security: 1. APP_AUTH 2. REFRESH_TOKEN
	 */
	private String appAuthUrl;
	private String refreshTokenUrl;

	/*
	 * Device Management: 
	 * 1. REGISTER_DEVICE 
	 * 2. MODIFY_DEVICE_INFO 
	 * 3. QUERY_DEVICE_ACTIVATION_STATUS 
	 * 4. DELETE_DEVICE 
	 * 5. DISCOVER_INDIRECT_DEVICE
	 * 6. REMOVE_INDIRECT_DEVICE
	 */
	private String registerDeviceUrl;
	public String getSelfCertPwd() {
		return selfCertPwd;
	}

	public void setSelfCertPwd(String selfCertPwd) {
		this.selfCertPwd = selfCertPwd;
	}

	public String getTrustCaPwd() {
		return trustCaPwd;
	}

	public void setTrustCaPwd(String trustCaPwd) {
		this.trustCaPwd = trustCaPwd;
	}

	private String modifyDeviceInfoUrl;
	private String queryDeviceActivationStatusUrl;
	private String deleteDeviceUrl;
	private String discoverIndirectDeviceUrl;
	private String removeIndirectDeviceUrl;

	/*
	 * Data Collection: 
	 * 1. QUERY_DEVICES 
	 * 2. QUERY_DEVICE_DATA 
	 * 3. QUERY_DEVICE_HISTORY_DATA 
	 * 4. QUERY_DEVICE_CAPABILITIES 
	 * 5. SUBSCRIBE_NOTIFYCATION
	 */
	private String queryDevicesUrl;
	private String queryDeviceDataUrl;
	private String queryDeviceHistoryDataUrl;
	private String queryDeviceCapabilitiesUrl;
	private String subscribeNotifycationUrl;

	/*
	 * Signaling Delivery： 
	 * 1. POST_ASYN_CMD 
	 * 2. QUERY_DEVICE_CMD 
	 * 3. UPDATE_ASYN_COMMAND 
	 * 4. CREATE_DEVICECMD_CANCEL_TASK 
	 * 5. QUERY_DEVICECMD_CANCEL_TASK
	 */
	private String postAsynCmdUrl;
	private String queryDeviceCmdUrl;
	private String updateAsynCommandUrl;
	private String createDeviceCmdCancelTaskUrl;
	private String queryDeviceCmdCancelTaskUrl;

	/*
	 * notify Type
	 * serviceInfoChanged|deviceInfoChanged|LocationChanged|deviceDataChanged|
	 * deviceDatasChanged
	 * deviceAdded|deviceDeleted|messageConfirm|commandRsp|deviceEvent|ruleEvent
	 */
	public static final String BIND_DEVICE = "bindDevice";
	public static final String DEVICE_ADDED = "deviceAdded";
	public static final String DEVICE_INFO_CHANGED = "deviceInfoChanged";
	public static final String DEVICE_DATA_CHANGED = "deviceDataChanged";
	public static final String DEVICE_DELETED = "deviceDeleted";
	public static final String MESSAGE_CONFIRM = "messageConfirm";
	public static final String SERVICE_INFO_CHANGED = "serviceInfoChanged";
	public static final String COMMAND_RSP = "commandRsp";
	public static final String DEVICE_EVENT = "deviceEvent";
	public static final String RULE_EVENT = "ruleEvent";
	public static final String DEVICE_DATAS_CHANGED = "deviceDatasChanged";
	public static final String SOFTWARE_UPGRADE_STATE_CHANGE = "swUpgradeStateChangeNotify";
	public static final String SOFTWARE_UPGRADE_RESULT = "swUpgradeResultNotify";
	public static final String FIREWARE_UPGRADE_STATE_CHANGE = "fwUpgradeStateChangeNotify";
	public static final String FIREWARE_UPGRADE_RESULT = "fwUpgradeResultNotify";
	
	/**
	 * 订阅url
	 */
	public String subscriptionsUrl;

	public String getBaseUrl() {
		return baseUrl;
	}

	public void setBaseUrl(String baseUrl) {
		this.baseUrl = baseUrl;
	}

	public String getAppId() {
		return appId;
	}

	public void setAppId(String appId) {
		this.appId = appId;
	}

	public String getSecret() {
		return secret;
	}

	public void setSecret(String secret) {
		this.secret = secret;
	}

	public String getCallbackBaseUrl() {
		return callbackBaseUrl;
	}

	public void setCallbackBaseUrl(String callbackBaseUrl) {
		this.callbackBaseUrl = callbackBaseUrl;
	}

	public String getReportCmdExecResultCallbackUrl() {
		return reportCmdExecResultCallbackUrl;
	}

	public void setReportCmdExecResultCallbackUrl(String reportCmdExecResultCallbackUrl) {
		this.reportCmdExecResultCallbackUrl = reportCmdExecResultCallbackUrl;
	}

	public String getSelfCertPath() {
		return selfCertPath;
	}

	public void setSelfCertPath(String selfCertPath) {
		this.selfCertPath = selfCertPath;
	}

	public String getTrustCaPath() {
		return trustCaPath;
	}

	public void setTrustCaPath(String trustCaPath) {
		this.trustCaPath = trustCaPath;
	}

	public String getAppAuthUrl() {
		return appAuthUrl;
	}

	public void setAppAuthUrl(String appAuthUrl) {
		this.appAuthUrl = appAuthUrl;
	}

	public String getRefreshTokenUrl() {
		return refreshTokenUrl;
	}

	public void setRefreshTokenUrl(String refreshTokenUrl) {
		this.refreshTokenUrl = refreshTokenUrl;
	}

	public String getRegisterDeviceUrl() {
		return registerDeviceUrl;
	}

	public void setRegisterDeviceUrl(String registerDeviceUrl) {
		this.registerDeviceUrl = registerDeviceUrl;
	}

	public String getModifyDeviceInfoUrl() {
		return modifyDeviceInfoUrl;
	}

	public void setModifyDeviceInfoUrl(String modifyDeviceInfoUrl) {
		this.modifyDeviceInfoUrl = modifyDeviceInfoUrl;
	}

	public String getQueryDeviceActivationStatusUrl() {
		return queryDeviceActivationStatusUrl;
	}

	public void setQueryDeviceActivationStatusUrl(String queryDeviceActivationStatusUrl) {
		this.queryDeviceActivationStatusUrl = queryDeviceActivationStatusUrl;
	}

	public String getDeleteDeviceUrl() {
		return deleteDeviceUrl;
	}

	public void setDeleteDeviceUrl(String deleteDeviceUrl) {
		this.deleteDeviceUrl = deleteDeviceUrl;
	}

	public String getDiscoverIndirectDeviceUrl() {
		return discoverIndirectDeviceUrl;
	}

	public void setDiscoverIndirectDeviceUrl(String discoverIndirectDeviceUrl) {
		this.discoverIndirectDeviceUrl = discoverIndirectDeviceUrl;
	}

	public String getRemoveIndirectDeviceUrl() {
		return removeIndirectDeviceUrl;
	}

	public void setRemoveIndirectDeviceUrl(String removeIndirectDeviceUrl) {
		this.removeIndirectDeviceUrl = removeIndirectDeviceUrl;
	}

	public String getQueryDevicesUrl() {
		return queryDevicesUrl;
	}

	public void setQueryDevicesUrl(String queryDevicesUrl) {
		this.queryDevicesUrl = queryDevicesUrl;
	}

	public String getQueryDeviceDataUrl() {
		return queryDeviceDataUrl;
	}

	public void setQueryDeviceDataUrl(String queryDeviceDataUrl) {
		this.queryDeviceDataUrl = queryDeviceDataUrl;
	}

	public String getQueryDeviceHistoryDataUrl() {
		return queryDeviceHistoryDataUrl;
	}

	public void setQueryDeviceHistoryDataUrl(String queryDeviceHistoryDataUrl) {
		this.queryDeviceHistoryDataUrl = queryDeviceHistoryDataUrl;
	}

	public String getQueryDeviceCapabilitiesUrl() {
		return queryDeviceCapabilitiesUrl;
	}

	public void setQueryDeviceCapabilitiesUrl(String queryDeviceCapabilitiesUrl) {
		this.queryDeviceCapabilitiesUrl = queryDeviceCapabilitiesUrl;
	}

	public String getSubscribeNotifycationUrl() {
		return subscribeNotifycationUrl;
	}

	public void setSubscribeNotifycationUrl(String subscribeNotifycationUrl) {
		this.subscribeNotifycationUrl = subscribeNotifycationUrl;
	}

	public String getPostAsynCmdUrl() {
		return postAsynCmdUrl;
	}

	public void setPostAsynCmdUrl(String postAsynCmdUrl) {
		this.postAsynCmdUrl = postAsynCmdUrl;
	}

	public String getQueryDeviceCmdUrl() {
		return queryDeviceCmdUrl;
	}

	public void setQueryDeviceCmdUrl(String queryDeviceCmdUrl) {
		this.queryDeviceCmdUrl = queryDeviceCmdUrl;
	}

	public String getUpdateAsynCommandUrl() {
		return updateAsynCommandUrl;
	}

	public void setUpdateAsynCommandUrl(String updateAsynCommandUrl) {
		this.updateAsynCommandUrl = updateAsynCommandUrl;
	}

	public String getCreateDeviceCmdCancelTaskUrl() {
		return createDeviceCmdCancelTaskUrl;
	}

	public void setCreateDeviceCmdCancelTaskUrl(String createDeviceCmdCancelTaskUrl) {
		this.createDeviceCmdCancelTaskUrl = createDeviceCmdCancelTaskUrl;
	}

	public String getQueryDeviceCmdCancelTaskUrl() {
		return queryDeviceCmdCancelTaskUrl;
	}

	public void setQueryDeviceCmdCancelTaskUrl(String queryDeviceCmdCancelTaskUrl) {
		this.queryDeviceCmdCancelTaskUrl = queryDeviceCmdCancelTaskUrl;
	}

	public String getDeviceAddedCallbackUrl() {
		return deviceAddedCallbackUrl;
	}

	public void setDeviceAddedCallbackUrl(String deviceAddedCallbackUrl) {
		this.deviceAddedCallbackUrl = deviceAddedCallbackUrl;
	}

	public String getDeviceInfoChangedCallbackUrl() {
		return deviceInfoChangedCallbackUrl;
	}

	public void setDeviceInfoChangedCallbackUrl(String deviceInfoChangedCallbackUrl) {
		this.deviceInfoChangedCallbackUrl = deviceInfoChangedCallbackUrl;
	}

	public String getDeviceDataChangedCallbackUrl() {
		return deviceDataChangedCallbackUrl;
	}

	public void setDeviceDataChangedCallbackUrl(String deviceDataChangedCallbackUrl) {
		this.deviceDataChangedCallbackUrl = deviceDataChangedCallbackUrl;
	}

	public String getDeviceDeletedCallbackUrl() {
		return deviceDeletedCallbackUrl;
	}

	public void setDeviceDeletedCallbackUrl(String deviceDeletedCallbackUrl) {
		this.deviceDeletedCallbackUrl = deviceDeletedCallbackUrl;
	}

	public String getMessageConfirmCallbackUrl() {
		return messageConfirmCallbackUrl;
	}

	public void setMessageConfirmCallbackUrl(String messageConfirmCallbackUrl) {
		this.messageConfirmCallbackUrl = messageConfirmCallbackUrl;
	}

	public String getServiceInfoChangedCallbackUrl() {
		return serviceInfoChangedCallbackUrl;
	}

	public void setServiceInfoChangedCallbackUrl(String serviceInfoChangedCallbackUrl) {
		this.serviceInfoChangedCallbackUrl = serviceInfoChangedCallbackUrl;
	}

	public String getCommandRspCallbackUrl() {
		return commandRspCallbackUrl;
	}

	public void setCommandRspCallbackUrl(String commandRspCallbackUrl) {
		this.commandRspCallbackUrl = commandRspCallbackUrl;
	}

	public String getDeviceEventCallbackUrl() {
		return deviceEventCallbackUrl;
	}

	public void setDeviceEventCallbackUrl(String deviceEventCallbackUrl) {
		this.deviceEventCallbackUrl = deviceEventCallbackUrl;
	}

	public String getRuleEventCallbackUrl() {
		return ruleEventCallbackUrl;
	}

	public void setRuleEventCallbackUrl(String ruleEventCallbackUrl) {
		this.ruleEventCallbackUrl = ruleEventCallbackUrl;
	}

	public String getDeviceDatasChangedCallbackUrl() {
		return deviceDatasChangedCallbackUrl;
	}

	public void setDeviceDatasChangedCallbackUrl(String deviceDatasChangedCallbackUrl) {
		this.deviceDatasChangedCallbackUrl = deviceDatasChangedCallbackUrl;
	}

	public String getSubscriptionsUrl() {
		return subscriptionsUrl;
	}

	public void setSubscriptionsUrl(String subscriptionsUrl) {
		this.subscriptionsUrl = subscriptionsUrl;
	}

}
