package com.hgzx.plp.ds.oc.config;

import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.context.annotation.Bean;

//@Configuration
public class RabbitConfig {
	
	// 贷后通平台消息队列
	public final static String PLP_DS_PKG_QUEUE_NAME = "plp_ds_pkg";
	public final static String PLP_DS_PKG_ROUTING_KEY = "plp_ds_pkg";
	public final static String PLP_DS_PKG_EXCHANGE = "plp_ds_pkg";
	
	/**  
     * durable="true" 持久化 rabbitmq重启的时候不需要创建新的队列  
       auto-delete 表示消息队列没有在使用时将被自动删除 默认是false  
       exclusive  表示该消息队列是否只在当前connection生效,默认是false
    */  
	@Bean
	public Queue plpQueue() {
		return new Queue(PLP_DS_PKG_QUEUE_NAME, true, false, false);  
	}
	
	/**  
    1.定义direct exchange，绑定queueTest  
    2.durable="true" rabbitmq重启的时候不需要创建新的交换机  
    3.
	  direct交换器相对来说比较简单，匹配规则为：如果路由键匹配，消息就被投送到相关的队列  
	  fanout交换器中没有路由键的概念，他会把消息发送到所有绑定在此交换器上面的队列中。  
	  topic交换器你采用模糊匹配路由键的原则进行转发消息到队列中  
	  key: queue在该direct-exchange中的key值，当消息发送给direct-exchange中指定key为设置值时，消息将会转发给queue参数指定 的消息队列  
    */  
	@Bean  
	public DirectExchange plpDirectExchange(){  
		return new DirectExchange(PLP_DS_PKG_EXCHANGE, true, false); 
	}
	
	@Bean
    Binding bindingPlpExchangeMessage() {
		return BindingBuilder.bind(plpQueue()).to(plpDirectExchange()).with(PLP_DS_PKG_ROUTING_KEY); 
    }

	@Bean
    public RabbitTemplate rabbitTemplate(ConnectionFactory connectionFactory) {
        RabbitTemplate template = new RabbitTemplate(connectionFactory);
        template.setMessageConverter(new Jackson2JsonMessageConverter());
        return template;
    }

    @Bean
    public SimpleRabbitListenerContainerFactory rabbitListenerContainerFactory(ConnectionFactory connectionFactory) {
        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        factory.setConnectionFactory(connectionFactory);
        factory.setMessageConverter(new Jackson2JsonMessageConverter());
        return factory;
    }
}
