package com.hgzx.plp.ds.oc.controller;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSON;
import com.hgzx.plp.ds.oc.model.OcRawData;
import com.hgzx.plp.ds.oc.service.DeviceDataService;

@Controller
@RequestMapping("/iocm")
public class OcCallbackController {
	
	private static final Logger logger = LoggerFactory.getLogger(OcCallbackController.class);

	@Autowired
	private DeviceDataService deviceDataService;
	
	/**
	 * 设备添加回调
	 * NA调用IoT平台的接口添加新设备，获取设备验证码，IoT平台使用该接口功能将该信息通知给NA，平台通知给Application。
	 * {
	 * 		"notifyType":"deviceAdded",
	 * 		"deviceId":"2f4e530b-f5b7-4704-89af-c5d5688ad164",
	 * 		"gatewayId":"2f4e530b-f5b7-4704-89af-c5d5688ad164",
	 * 		"nodeType":"GATEWAY",
	 * 		"deviceInfo":
	 * 		{
	 * 			"nodeId":"863703036230469",
	 * 			"name":"",
	 * 			"manufacturerId":"creditstate",
	 * 			"manufacturerName":"creditstate",
	 * 			"location":"",
	 * 			"deviceType":"hgzxim",
	 * 			"model":"hgzxim01",
	 * 			"protocolType":"CoAP",
	 * 			"status":"OFFLINE",
	 * 			"statusDetail":"NOT_ACTIVE"
	 * 		}
	 * }
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */

	@RequestMapping(value = "/callback/deviceAdded", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvDeviceAddedNotify(@RequestBody String notifyMessage) {
		logger.info("--- add-device callback: {} ---", notifyMessage);
		try {
			OcRawData notifyData = JSON.parseObject(notifyMessage, OcRawData.class);
			logger.info("send device-added-data to mq {}", notifyData);
			//mqSender.send(notifyData);
		} catch(Exception e) {
			e.printStackTrace();
		}
		return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 设备删除回调
	 * IoT平台接收到删除非直连设备，IoT平台使用该接口功能将该信息通知给Application
	 * {	
	 * 		"notifyType":"deviceDeleted",
	 * 		"deviceId":"2f4e530b-f5b7-4704-89af-c5d5688ad164",
	 * 		"gatewayId":"2f4e530b-f5b7-4704-89af-c5d5688ad164"
	 * }
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/deviceDeleted", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvDeviceDeletedNotify(@RequestBody String notifyMessage) {
		logger.info("--- deleted-device callback: {} ---", notifyMessage);
		try {
			OcRawData notifyData = JSON.parseObject(notifyMessage, OcRawData.class);
			logger.info("send deleted-device-data to mq {}", notifyData);
			//mqSender.send(notifyData);
			deviceDataService.insert(notifyData.getOcData());
		} catch(Exception e) {
			e.printStackTrace();
		}
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 设备信息变化回调
	 * IoT平台接收到设备信息变化（静态信息，如设备名称、厂商id），IoT平台使用该接口功能将该信息通知给Application。
	 * {
	 * 		"notifyType":"deviceInfoChanged",
	 * 		"deviceId":"e56ace1e-99b6-4190-bce2-3b6c6fccdfb7",
	 * 		"gatewayId":"e56ace1e-99b6-4190-bce2-3b6c6fccdfb7",
	 * 		"deviceInfo":
	 * 		{
	 * 			"nodeId":null,
	 * 			"name":null,
	 * 			"description":null,
	 * 			"manufacturerId":null,
	 * 			"manufacturerName":null,
	 * 			"mac":null,
	 * 			"location":null,
	 * 			"deviceType":null,
	 * 			"model":null,
	 * 			"swVersion":null,
	 * 			"fwVersion":null,
	 * 			"hwVersion":null,
	 * 			"protocolType":null,
	 * 			"bridgeId":null,
	 * 			"status":"ONLINE",
	 * 			"statusDetail":"NONE",
	 * 			"mute":null,
	 * 			"supportedSecurity":null,
	 * 			"isSecurity":null,
	 * 			"signalStrength":null,
	 * 			"sigVersion":null,
	 * 			"serialNumber":null,
	 * 			"batteryLevel":null
	 * 		}
	 * }
	 * 
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/deviceInfoChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvDeviceInfoChangedNotify(@RequestBody String notifyMessage) {
		logger.info("--- deviceInfo-changed callback: {} ---", notifyMessage);
		try {
			OcRawData notifyData = JSON.parseObject(notifyMessage, OcRawData.class);
			System.out.println(notifyMessage);
			logger.info("send deviceInfo-changed-data to mq {}", notifyData);

			//mqSender.send(notifyData);
		} catch(Exception e) {
			e.printStackTrace();
		}
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	 /**
	 * 设备绑定激活
	 * 即设备初始登陆通知，网关设备在平台注册绑定（网关设备在平台创建对应设备的信息并获取密码的过程），平台通知给NA。
	 * 
	 * {
	 * 		"notifyType":"bindDevice",
	 * 		"deviceId":"e56ace1e-99b6-4190-bce2-3b6c6fccdfb7",
	 * 		"resultCode":"succeeded",
	 * 		"deviceInfo":
	 * 		{
	 * 			"nodeId":"863703036230469",
	 * 			"name":"",
	 * 			"description":null,
	 * 			"manufacturerId":"creditstate",
	 * 			"manufacturerName":"creditstate",
	 * 			"mac":null,
	 * 			"location":"",
	 * 			"deviceType":"hgzxim",
	 * 			"model":"hgzxim01",
	 * 			"swVersion":null,
	 * 			"fwVersion":null,
	 * 			"hwVersion":null,
	 * 			"protocolType":"CoAP",
	 * 			"bridgeId":null,
	 * 			"status":"OFFLINE",
	 * 			"statusDetail":"NOT_ACTIVE",
	 * 			"mute":null,
	 * 			"supportedSecurity":null,
	 * 			"isSecurity":null,
	 * 			"signalStrength":null,
	 * 			"sigVersion":null,
	 * 			"serialNumber":null,
	 * 			"batteryLevel":null
	 *		}
	 * }
	 * 
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/deviceActivated", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvDeviceActivatedNotify(@RequestBody String notifyMessage) {
		logger.info("--- device-activated-data callback: {} ---", notifyMessage);
		try {
			OcRawData notifyData = JSON.parseObject(notifyMessage, OcRawData.class);
			logger.info("--- send device-activated-info data to mq: {} ---", notifyData);
			//mqSender.send(notifyData);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}

	/**
	 * 消息确认回调
	 * IoT平台接收到网关的确认消息（如下发命令给网关，网关确认接收到该消息），将该消息通知给Application。
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/messageConfirm", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvMessageConfirmNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- message-confirm callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 服务变化回调
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/serviceInfoChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvServiceInfoChangedNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- service-info-changed callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 响应命令回调
	 * IoT平台接收到设备（包括网关设备和普通设备）的响应命令（如下发命令给设备，设备执行命令后，发送响应命令给平台，如视频呼叫、视频录制，抓图等），将该消息通知给Application。
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/commandRsp", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvCommandRspNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- command response data: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 命令执行结果回调--设备反馈的执行结果
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/reportCmdExecResult", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvCommandExecResultRspNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- command-Execute-Result callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 设备事件回调
	 * IoT平台接收到设备发出的事件（如设备UDS存储空间不足事件），将该消息通知给Application。
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/deviceEvent", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvDeviceEventNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- device-event callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 规则事件回调
	 * IoT平台规则触发后生成对应的规则事件通知NA。
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/callback/ruleEvent", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvRuleEventNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- rule-event callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 设备数据变化回调
	 * IoT平台接收到设备数据变化（动态变化，如，服务属性值的变化），IoT平台使用该接口功能将该信息通知给Application。
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 * {
	 * 		"notifyType":"deviceDataChanged",
	 * 		"deviceId":"13f65cf7-00ad-4c9b-9ab2-1ca5e7bec290",
	 * 		"gatewayId":"13f65cf7-00ad-4c9b-9ab2-1ca5e7bec290",
	 * 		"requestId":null,
	 * 		"service":
	 * 		{
	 * 			"serviceId":"upData",
	 * 			"serviceType":"upData",
	 * 			"data":
	 * 			{
	 * 				"data":"1805210001,1.0.0,4057953900,23,000,863703036230675,460111174770102,00,0000000000,0000000000,0000000000,0000000001,0"
	 * 			},
	 * 			"eventTime":"20181019T023341Z"
	 * 		}
	 * 	}
	 */
	@RequestMapping(value = "/callback/newDeviceDataChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvDeviceDataChangedNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- update-device-data callback: {} ---", notifyMessage);
		try {
			OcRawData rawData = JSON.parseObject(notifyMessage, OcRawData.class);
			logger.info("--- send update data to mq: {} ---", rawData.getOcData());
			//mqSender.send(rawData);
			deviceDataService.insertDeviceData(rawData.getOcData());
//			deviceDataService.insert(rawData.getOcData());
		} catch(Exception e) {
			e.printStackTrace();
		}
		return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	/**
	 * 批量设备数据变化后消息通知回调
	 * @param notifyMessage
	 * @return
	 * @throws IOException
	 * {
	 * 		"notifyType":"deviceDatasChanged",
	 * 		"requestId":null,
	 * 		"deviceId":"13f65cf7-00ad-4c9b-9ab2-1ca5e7bec290",
	 * 		"gatewayId":"13f65cf7-00ad-4c9b-9ab2-1ca5e7bec290",
	 * 		"services":
	 * 		[
	 * 			{
	 * 				"serviceId":"upData",
	 * 				"serviceType":"upData",
	 * 				"data":
	 * 				{
	 * 					"data":"1805210001,1.0.0,4058184700,22,000,863703036230675,460111174770102,0000,0000000004,0000000000,0000000000,0000000001,0"
	 * 				},
	 * 				"eventTime":"20181019T060349Z"
	 * 			}
	 * 		]
	 *	}
	 */
	@RequestMapping(value = "/callback/deviceDatasChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvDeviceDatasChangedNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- update-device-datas callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	@RequestMapping(value = "/callback/swUpgradeStateChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvSwUpgradeStateChangedNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- software upgrade state change callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	@RequestMapping(value = "/callback/swUpgradeResultChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvSwUpgradeResultNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- fireware upgrade result callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	@RequestMapping(value = "/callback/fwUpgradeStateChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvFwUpgradeStateChangedNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- fireware upgrade state change callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
	@RequestMapping(value = "/callback/fwUpgradeResultChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public ResponseEntity<HttpStatus> recvFwUpgradeResultNotify(@RequestBody String notifyMessage) throws IOException {
		logger.info("--- fireware upgrade result callback: {} ---", notifyMessage);
    	return new ResponseEntity<HttpStatus>(HttpStatus.OK);
	}
	
}