package com.hgzx.plp.ds.oc.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSON;
import com.hgzx.plp.ds.oc.model.OcDeviceInfo;
import com.hgzx.plp.ds.oc.model.OcError;
import com.hgzx.plp.ds.oc.model.OcRsp;
import com.hgzx.plp.ds.oc.model.ResultData;
import com.hgzx.plp.ds.oc.service.OcDeviceService;

/**
 * 设备管理北向服务接口
 * @author 柏耀明
 */
@Controller
@RequestMapping("/iocm")
public class OcDeviceController extends BaseController {
	
	private static final Logger logger = LoggerFactory.getLogger(OcDeviceController.class);
	
	@Autowired
	private OcDeviceService deviceService;
	
	/**
	 * 注册设备 200为成功返回码
	 * @param verifyCode 验证码
	 * @param nodeId 唯一码
	 * @return
	 */
	@RequestMapping(value = "/device/reg", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String regDevice(String verifyCode, String nodeId) {
		ResultData<OcDeviceInfo> result = new ResultData<OcDeviceInfo>();
		OcRsp rsp = new OcRsp();
		try {
			rsp = deviceService.registerDevice(verifyCode, nodeId, 0);//0 标识不超时
			OcDeviceInfo deviceInfo = new OcDeviceInfo();
			if(rsp.getStatusCode() == 200) {
				deviceInfo = rsp.getData(OcDeviceInfo.class);
				result.setResult(deviceInfo);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result.setSuccess(false);
				result.setResultMsg(ocError.getError_desc());
				result.setResultCode(ocError.getError_code());
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		logger.info("--registry device-- verifyCode:{},nodeId:{},Response:{}", verifyCode, nodeId, rsp);
		return JSON.toJSONString(result);
	}
	
	/**
	 * 更新设备 204为成功返回码
	 * @param deviceInfo
	 * @return
	 */
	@RequestMapping(value = "/device/update", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String updateDevice(OcDeviceInfo deviceInfo) {
		OcRsp rsp = new OcRsp();
		ResultData<OcDeviceInfo> result = new ResultData<OcDeviceInfo>();
		try {
			rsp = deviceService.modifyDeviceInfo(deviceInfo);
			if(rsp.getStatusCode() != 204) {
				OcError ocError = rsp.getData(OcError.class);
				result.setResultCode(ocError.getError_code());
				result.setResultCode(ocError.getError_desc());
				result.setSuccess(false);
			} else {
				result.setResult(deviceInfo);
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		logger.info("--update device info--: {},Response:{}", deviceInfo, rsp);
		return JSON.toJSONString(result);
	}
	
	/**
	 * 删除设备 204为成功返回码
	 * @param deviceId
	 * @return
	 */
	@RequestMapping(value = "/device/delete", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String deleteDevice(String deviceId) {
		ResultData<OcDeviceInfo> result = new ResultData<OcDeviceInfo>();
		OcRsp rsp = new OcRsp();
		try {
			rsp = deviceService.deleteDirectlyConnectedDevice(deviceId);
			if(rsp.getStatusCode() != 204) {
				OcDeviceInfo di = new OcDeviceInfo();
				di.setDeviceId(deviceId);
				result.setResult(di);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result.setResultCode(ocError.getError_code());
				result.setResultCode(ocError.getError_desc());
				result.setSuccess(false);
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		logger.info("--delete device-- deviceId:{},Response:{}", deviceId, rsp);
		return JSON.toJSONString(result);
	}
	
	/**
	 * 查询设备激活状态 200为成功返回码
	 * @param deviceId
	 * @return
	 */
	@RequestMapping(value = "/device/activeStatus", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String queryDeviceStatus(String deviceId) {
		OcRsp rsp = new OcRsp();
		ResultData<OcDeviceInfo> result = new ResultData<OcDeviceInfo>();
		try {
			rsp = deviceService.queryDeviceActivationStatus(deviceId);
			if(rsp.getStatusCode() == 200) {
				OcDeviceInfo di = rsp.getData(OcDeviceInfo.class);
				result.setResult(di);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result.setResultCode(ocError.getError_code());
				result.setResultMsg(ocError.getError_desc());
				result.setSuccess(false);
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		logger.info("--delete device-- deviceId:{},Response:{}", deviceId, rsp);
		return JSON.toJSONString(result);
	}
	
}