package com.hgzx.plp.ds.oc.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSON;
import com.hgzx.plp.ds.oc.config.OcConfig;
import com.hgzx.plp.ds.oc.model.OcError;
import com.hgzx.plp.ds.oc.model.OcRsp;
import com.hgzx.plp.ds.oc.model.ResultData;
import com.hgzx.plp.ds.oc.service.OcSubscribeService;

@Controller
@RequestMapping("/iocm")
public class OcNotifySubscriptController {
	
	private static final Logger logger = LoggerFactory.getLogger(OcNotifySubscriptController.class);
	
	@Autowired
	private OcSubscribeService subscribeService;
	
	/**
	 * 订阅数据变化 回调地址
	 */
	@RequestMapping(value = "/subscript/deviceBinded", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String bindDeviceSubscript(String callbackUrl) {
		logger.info("--- subscript-devcie-bind-callback url: {} ---", callbackUrl);
		ResultData<Object> result = new ResultData<Object>();
		OcRsp rsp = new OcRsp();
		try {
			rsp = subscribeService.subscribe(OcConfig.BIND_DEVICE, callbackUrl);
			if(rsp.getStatusCode() == 201) {
				result.setSuccess(true);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result.setSuccess(false);
				result.setResultMsg(ocError.getError_desc());
				result.setResultCode(ocError.getError_code());
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		return JSON.toJSONString(result, true);
	}
	
	/**
	 * 订阅数据变化 回调地址
	 */
	@RequestMapping(value = "/subscript/deviceDataChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String deviceDataChangedSubscript(String callbackUrl) {
		logger.info("--- subscript-Data-Changed-callback url: {} ---", callbackUrl);
		ResultData<Object> result = new ResultData<Object>();
		OcRsp rsp = new OcRsp();
		try {
			rsp = subscribeService.deviceDataChanged(callbackUrl);
			if(rsp.getStatusCode() == 201) {
				result.setSuccess(true);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result.setSuccess(false);
				result.setResultMsg(ocError.getError_desc());
				result.setResultCode(ocError.getError_code());
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		return JSON.toJSONString(result, true);
	}
	
	/**
	 * 订阅数据批量变化 回调地址
	 */
	@RequestMapping(value = "/subscript/deviceDatasChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String deviceDatasChangedSubscript(String callbackUrl) {
		logger.info("--- subscript-Datas-Changed-callback-url: {} ---", callbackUrl);
		ResultData<Object> result = new ResultData<Object>();
		OcRsp rsp = new OcRsp();
		try {
			rsp = subscribeService.deviceDataChanged(callbackUrl);
			if(rsp.getStatusCode() == 201) {
				result.setSuccess(true);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result.setSuccess(false);
				result.setResultMsg(ocError.getError_desc());
				result.setResultCode(ocError.getError_code());
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		return JSON.toJSONString(result, true);
	}
	
	/**
	 * 订阅设备信息变化 回调地址
	 */
	@RequestMapping(value = "/subscript/deviceInfoChanged", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String deviceInfoChangedSubscript(String callbackUrl) {
		logger.info("--- subscript-device-info-change-callback-url: {} ---", callbackUrl);
		ResultData<Object> result = new ResultData<Object>();
		OcRsp rsp = new OcRsp();
		try {
			rsp = subscribeService.deviceInfoChanged(callbackUrl);
			if(rsp.getStatusCode() == 201) {
				result.setSuccess(true);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result.setSuccess(false);
				result.setResultMsg(ocError.getError_desc());
				result.setResultCode(ocError.getError_code());
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		return JSON.toJSONString(result, true);
	}
	
	/**
	 * 订阅设备信息变化 回调地址
	 */
	@RequestMapping(value = "/subscript/query", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String querySubscript(String notifyType, String pageNo, String pageSize) {
		logger.info("--- query subscript -- notify:{}, pageNo:{}, pageSize:{} ---", notifyType, pageNo, pageSize);
		ResultData<Object> result = new ResultData<Object>();
		OcRsp rsp = new OcRsp();
		try {
			rsp = subscribeService.querySubscription(notifyType, pageNo, pageSize);
			if(rsp.getStatusCode() == 200) {
				result.setSuccess(true);
				result.setResult(rsp.getResponseBody());
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result.setSuccess(false);
				result.setResultMsg(ocError.getError_desc());
				result.setResultCode(ocError.getError_code());
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		return JSON.toJSONString(result);
	}
	
}