package com.hgzx.plp.ds.oc.model;

import java.io.Serializable;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

/**
 * oc平台通信报文
 * @author 柏耀明
 */
public class OcRawData implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * 通知类型
	 */
	private String notifyType;
	
	/**
	 * oc平台设备id
	 */
	private String deviceId;
	
	/**
	 * oc平台设备网关id
	 */
	private String gatewayId;
	
	/**
	 * 请求id
	 */
	private String requestId;
	
	/**
	 * 服务数据
	 */
	private String service;
	
	/**
	 * 设备信息
	 */
	private String deviceInfo;
	
	public static void main(String[] args) throws ParseException {
		//String t = "{\"notifyType\":\"deviceDataChanged\",\"deviceId\":\"13f65cf7-00ad-4c9b-9ab2-1ca5e7bec290\",\"gatewayId\":\"13f65cf7-00ad-4c9b-9ab2-1ca5e7bec290\",\"requestId\":null,\"service\":{\"serviceId\":\"upData\",\"serviceType\":\"upData\",\"data\":{\"data\":\"1805210001,1.0.0,4057953900,23,000,863703036230675,460111174770102,00,0000000000,0000000000,0000000000,0000000001,0\"},\"eventTime\":\"20181019T023341Z\"}}";		
		//String t1 = "{\"notifyType\":\"deviceDeleted\",\"deviceId\":\"2f4e530b-f5b7-4704-89af-c5d5688ad164\",\"gatewayId\":\"2f4e530b-f5b7-4704-89af-c5d5688ad164\"}";
		//String t2 = "{\"notifyType\":\"deviceAdded\",\"deviceId\":\"2f4e530b-f5b7-4704-89af-c5d5688ad164\",\"gatewayId\":\"2f4e530b-f5b7-4704-89af-c5d5688ad164\",\"nodeType\":\"GATEWAY\",\"deviceInfo\":{\"nodeId\":\"863703036230469\",\"name\":\"hello\",\"manufacturerId\":\"creditstate\",\"manufacturerName\":\"creditstate\",\"location\":\"wuhu\",\"deviceType\":\"hgzxim\",\"model\":\"hgzxim01\",\"protocolType\":\"CoAP\",\"status\":\"OFFLINE\",\"statusDetail\":\"NOT_ACTIVE\"}}";
		String t3 = "{\"notifyType\":\"deviceInfoChanged\",\"deviceId\":\"e56ace1e-99b6-4190-bce2-3b6c6fccdfb7\",\"gatewayId\":\"e56ace1e-99b6-4190-bce2-3b6c6fccdfb7\",\"deviceInfo\":{\"nodeId\":null,\"name\":null,\"description\":null,\"manufacturerId\":null,\"manufacturerName\":null}}";
		OcRawData rd = JSON.parseObject(t3, OcRawData.class);
		System.out.println(rd.getOcDeviceInfo());
	}
	
	/**
	 * 获取载荷数据
	 * \"service\":{\"serviceId\":\"upData\",\"serviceType\":\"upData\",\"data\":{\"data\":\"1805210001,1.0.0,4057953900,23,000,863703036230675,460111174770102,00,0000000000,0000000000,0000000000,0000000001,0\"},\"eventTime\":\"20181019T023341Z\"}}";			 * @return
	 */
	public OcJhtData getOcData() {
		if(StringUtils.isEmpty(this.getService())) {
			return null;
		}
		
		try {
			JSONObject serviceJo = JSON.parseObject(this.getService());
			JSONObject serviceDataJo = serviceJo.getJSONObject("data");
			if(!serviceDataJo.isEmpty() && serviceDataJo != null) {
				OcJhtData ocData = new OcJhtData();
				ocData.setDeviceSn(serviceDataJo.getString("deviceSn"));
				ocData.setMessageId(serviceDataJo.getString("msgId"));
				ocData.setDeviceStatus(serviceDataJo.getString("disassembleStatus"));
				ocData.setDeviceBatteryLevel(serviceDataJo.getString("batteryLevel"));
				ocData.setPositionStatus(serviceDataJo.getString("positionStatus"));
				double latitude = (serviceDataJo.getDouble("latitude")/100);
				double longitude = (serviceDataJo.getDouble("longitude")/100);
				DecimalFormat df = new DecimalFormat( "0.000000");
				System.out.println(latitude);
				System.out.println(longitude);
				ocData.setGpsLatitude(df.format(latitude));
				ocData.setGpsLongitude(df.format(longitude));
				ocData.setGpsTime(serviceDataJo.getString("infoTime"));
				ocData.setGpsSpeed(serviceDataJo.getString("velocity"));
				ocData.setGpsAngle(serviceDataJo.getString("direction"));
				ocData.setGpsAltitude(serviceDataJo.getString("altitude"));
				ocData.setImsi(serviceDataJo.getString("imsi"));
				ocData.setBaseStationId(serviceDataJo.getString("baseStationId"));
				ocData.setPci(serviceDataJo.getString("baseStationPci"));
				ocData.setCsq(serviceDataJo.getString("baseStationSignal"));
				ocData.setFirmwareVersion(serviceDataJo.getString("firmwareVer"));
				ocData.setAwakenInterval(serviceDataJo.getString("awakeInterval"));
				ocData.setReserve(serviceDataJo.getString("result"));
				SimpleDateFormat sfmt = new SimpleDateFormat();
				sfmt.applyPattern("yyyyMMddHHmmss");
				Date dataTime = sfmt.parse(serviceJo.getString("eventTime").replace("T", "").replace("Z", ""));
				sfmt.applyPattern("yyyy-MM-dd HH:mm:ss");
				ocData.setDataTime(sfmt.format(dataTime));
			    ocData.setDataRevTime(sfmt.format(new Date()));
				return ocData;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public OcDeviceInfo getOcDeviceInfo() {
		if(StringUtils.isEmpty(this.getDeviceInfo())) {
			OcDeviceInfo retDeviceInfo = new OcDeviceInfo();
			retDeviceInfo.setDeviceId(this.getDeviceId());
			return retDeviceInfo;
		}
		try {
			JSONObject deviceInfoJo = JSON.parseObject(this.getDeviceInfo());
			OcDeviceInfo retDeviceInfo = new OcDeviceInfo();
			if(!deviceInfoJo.isEmpty()) {
				retDeviceInfo.setNodeId(deviceInfoJo.getString("nodeId"));
				retDeviceInfo.setName(deviceInfoJo.getString("name"));
				retDeviceInfo.setDeviceType(deviceInfoJo.getString("deviceType"));
				retDeviceInfo.setManufacturerId(deviceInfoJo.getString("manufacturerId"));
				retDeviceInfo.setManufacturerName(deviceInfoJo.getString("manufacturerName"));
				retDeviceInfo.setModel(deviceInfoJo.getString("model"));
				retDeviceInfo.setProtocolType(deviceInfoJo.getString("protocolType"));
				retDeviceInfo.setLocation(deviceInfoJo.getString("location"));
				retDeviceInfo.setDeviceId(this.getDeviceId());
				return retDeviceInfo;
			} else {
				retDeviceInfo.setDeviceId(this.getDeviceId());
				return retDeviceInfo;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public String getNotifyType() {
		return notifyType;
	}

	public void setNotifyType(String notifyType) {
		this.notifyType = notifyType;
	}

	public String getDeviceId() {
		return deviceId;
	}

	public void setDeviceId(String deviceId) {
		this.deviceId = deviceId;
	}

	public String getGatewayId() {
		return gatewayId;
	}

	public void setGatewayId(String gatewayId) {
		this.gatewayId = gatewayId;
	}

	public String getRequestId() {
		return requestId;
	}

	public void setRequestId(String requestId) {
		this.requestId = requestId;
	}

	public String getService() {
		return service;
	}

	public void setService(String service) {
		this.service = service;
	}

	public String getDeviceInfo() {
		return deviceInfo;
	}

	public void setDeviceInfo(String deviceInfo) {
		this.deviceInfo = deviceInfo;
	}

	@Override
	public String toString() {
		return "OcRawData [notifyType=" + notifyType + ", deviceId=" + deviceId + ", gatewayId="
				+ gatewayId + ", requestId=" + requestId + ", service=" + service + ", deviceInfo=" + deviceInfo + "]";
	}
	
}
