package com.hgzx.plp.ds.oc.service;

import java.util.HashMap;
import java.util.Map;

import org.apache.http.HttpResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.hgzx.plp.ds.oc.common.utils.HttpsUtil;
import com.hgzx.plp.ds.oc.common.utils.JsonUtil;
import com.hgzx.plp.ds.oc.common.utils.StreamClosedHttpResponse;
import com.hgzx.plp.ds.oc.config.OcConfig;
import com.hgzx.plp.ds.oc.model.OcRsp;

@Service
public class OcCommandService {
	
	private static final Logger logger = LoggerFactory.getLogger(OcCommandService.class);
	
	@Autowired
    private OcTokenService tokenService;
	
	@Autowired
	private OcConfig ocConfig;
	
	private HttpsUtil getHttpsClient() throws Exception {
		HttpsUtil httpsUtil = new HttpsUtil();
		
		String certPath = ocConfig.getSelfCertPath();
		String certPwd = ocConfig.getSelfCertPwd();
		
		String caPath = ocConfig.getTrustCaPath();
		String caPwd = ocConfig.getTrustCaPwd();
		
        httpsUtil.createSSLClient(certPath, certPwd, caPath, caPwd);
        return httpsUtil;
	}
	
	public OcRsp createDeviceCmdCancelTaskV4(String deviceId) throws Exception {
		
		HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();
		
        String urlUpdateAsynCommand = ocConfig.getCreateDeviceCmdCancelTaskUrl();// Constant.CREATE_DEVICECMD_CANCEL_TASK;
        
        Map<String, Object> paraCreateDeviceCmdCancelTask = new HashMap<String, Object>();
        paraCreateDeviceCmdCancelTask.put("deviceId", deviceId);
        
        String jsonRequest = JsonUtil.jsonObj2Sting(paraCreateDeviceCmdCancelTask);
                
        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doPostJsonGetStatusLine(urlUpdateAsynCommand, header, jsonRequest);
        
        logger.debug("Update Asyn Command, deviceId: {}, response content: status:{}, content:{}", 
				deviceId, response.getStatusLine().getStatusCode(), response.getContent());
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), response.getContent());
        return rsp;
    }

    public OcRsp postAsynCommandV4(String serviceId, String deviceId, String method, String paramJson, Integer expireTime) throws Exception {
    	HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();
        String urlPostAsynCmd = ocConfig.getPostAsynCmdUrl();// Constant.POST_ASYN_CMD;

        String callbackUrl = ocConfig.getReportCmdExecResultCallbackUrl();// Constant.REPORT_CMD_EXEC_RESULT_CALLBACK_URL;
        ObjectNode paras = JsonUtil.convertObject2ObjectNode(paramJson);
      
        Map<String, Object> paramCommand = new HashMap<String, Object>();
        paramCommand.put("serviceId", serviceId);
        paramCommand.put("method", method);
        paramCommand.put("paras", paras);
        
        Map<String, Object> paramPostAsynCmd = new HashMap<String, Object>();
        paramPostAsynCmd.put("deviceId", deviceId);
        paramPostAsynCmd.put("command", paramCommand);
        paramPostAsynCmd.put("callbackUrl", callbackUrl);
        if(expireTime != null) {
        	paramPostAsynCmd.put("expireTime", expireTime);
        }
        String jsonRequest = JsonUtil.jsonObj2Sting(paramPostAsynCmd);
                
        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        HttpResponse response = httpsUtil.doPostJson(urlPostAsynCmd, header, jsonRequest);

        String responseBody = httpsUtil.getHttpResponseBody(response);
        
        logger.debug("Post Asyn Command, deviceId: {}, response content: status:{}, content:{}", 
				deviceId, response.getStatusLine().getStatusCode(), responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
    }
    
	public String queryAsynCommandsV4() throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();

        //Please make sure that the following parameter values have been modified in the Constant file.
        String queryDeviceCmdUrl = ocConfig.getQueryDeviceCmdUrl();// Constant.QUERY_DEVICE_CMD;
        
        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doGetWithParasGetStatusLine(queryDeviceCmdUrl, null, header);
        
        logger.debug("Post Asyn Command, response content: status:{}, content:{}", 
				response.getStatusLine().getStatusCode(), response.getContent());
        return response.getContent();
    }
	
	
	public String queryDeviceCmdCancelTaskV4() throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();
        String url = ocConfig.getQueryDeviceCmdUrl();// Constant.QUERY_DEVICECMD_CANCEL_TASK;
        
        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doGetWithParasGetStatusLine(url, null, header);
        
        logger.debug("Query Asyn Commands, response content: status:{}, content:{}", 
				response.getStatusLine().getStatusCode(), response.getContent());
        return response.getContent();
    }

	public String updateAsynCommandV4(String commandId) throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();

        String url = ocConfig.getUpdateAsynCommandUrl(); //Constant.UPDATE_ASYN_COMMAND;
        url = String.format(url, commandId);

        //Currently only supports Modify the status of device command from PENDING to CANCELED.
        String status = "CANCELED";
        
        Map<String, Object> paraUpdateDeviceCommandReq = new HashMap<>();
        paraUpdateDeviceCommandReq.put("status", status);
        
        String jsonRequest = JsonUtil.jsonObj2Sting(paraUpdateDeviceCommandReq);
                
        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doPutJsonGetStatusLine(url, header, jsonRequest);
        
        logger.debug("Update Asyn Command, commandId:{}, response content: status:{}, content:{}", 
        		commandId, response.getStatusLine().getStatusCode(), response.getContent());
        return response.getContent();
    }
	
}
