package com.hgzx.plp.ds.oc.service;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hgzx.plp.ds.oc.common.utils.HttpsUtil;
import com.hgzx.plp.ds.oc.common.utils.StreamClosedHttpResponse;
import com.hgzx.plp.ds.oc.config.OcConfig;

@Service
public class OcDataCollectService {
	
	private static final Logger logger = LoggerFactory.getLogger(OcDataCollectService.class);
	
	@Autowired
    private OcTokenService tokenService;
	
	@Autowired
	private OcConfig ocConfig;
	
	private HttpsUtil getHttpsClient() throws Exception {
		HttpsUtil httpsUtil = new HttpsUtil();
		
		String certPath = ocConfig.getSelfCertPath();
		String certPwd = ocConfig.getSelfCertPwd();
		
		String caPath = ocConfig.getTrustCaPath();
		String caPwd = ocConfig.getTrustCaPwd();
		
        httpsUtil.createSSLClient(certPath, certPwd, caPath, caPwd);
        return httpsUtil;
	}
	
	/**
	 * 查询设备服务能力
	 * @param deviceId 设备Id
	 * @param gatewayId 网关ID
	 * @return
	 * @throws Exception
	 */
	public String queryDeviceCapabilities(String deviceId, String gatewayId) throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String url = ocConfig.getQueryDeviceCapabilitiesUrl();

        Map<String, String> params = new HashMap<String, String>();
        params.put("deviceId", deviceId);
        params.put("gatewayId", gatewayId);

        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doGetWithParasGetStatusLine(url, params, header);
        logger.debug("query device capablilities, response content: status:{}, content:{}", 
        		response.getStatusLine().getStatusCode(), response.getContent());
        return response.getContent();
    }
	
	/**
	 * 获取设备数据
	 * @param deviceId 设备id
	 * @return
	 * @throws Exception
	 */
    public String queryDeviceData(String deviceId) throws Exception {
    	HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String url = ocConfig.getQueryDeviceDataUrl() + "/" + deviceId;

        Map<String, String> params = new HashMap<String, String>();
        params.put("appId", appId);

        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doGetWithParasGetStatusLine(url, params, header);
        logger.debug("Query Device Data, response content: status:{}, content:{}", 
        		response.getStatusLine().getStatusCode(), response.getContent());
        return response.getContent();
    }
    
    /**
     * 获得设备历史数据
     * @param deviceId 设备id
     * @param gatewayId 网关id
     * @return
     */
    public String queryDeviceHistoryData(String deviceId, String gatewayId) throws Exception {
    	HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String url = ocConfig.getQueryDeviceHistoryDataUrl();

        Map<String, String> params = new HashMap<>();
        params.put("deviceId", deviceId);
        params.put("gatewayId", gatewayId);

        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doGetWithParasGetStatusLine(url, params, header);
        logger.debug("Query Device History Data, response content: status:{}, content:{}", 
        		response.getStatusLine().getStatusCode(), response.getContent());
        return response.getContent(); 
    }

    /**
     * 查询设备信息
     * @param pageNo    页面大小
     * @param pageSize  当前页
     * @return
     * @throws Exception
     */
    public String queryDevices(Integer pageNo, Integer pageSize) throws Exception {
    	HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String url = ocConfig.getQueryDevicesUrl();

        Map<String, String> params = new HashMap<>();
        params.put("appId", appId);
        params.put("pageNo", pageNo.toString());
        params.put("pageSize", pageSize.toString());

        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doGetWithParasGetStatusLine(url, params, header);
        logger.debug("Query Devices, response content: status:{}, content:{}", 
        		response.getStatusLine().getStatusCode(), response.getContent());
        
        return response.getContent();
    }
	
}
