package com.hgzx.plp.ds.oc.service;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hgzx.plp.ds.oc.common.utils.HttpsUtil;
import com.hgzx.plp.ds.oc.common.utils.JsonUtil;
import com.hgzx.plp.ds.oc.config.OcConfig;
import com.hgzx.plp.ds.oc.model.OcRsp;

@Service
public class OcSubscribeService {
	
	private static final Logger logger = LoggerFactory.getLogger(OcSubscribeService.class);
	
	@Autowired
    private OcTokenService tokenService;
	
	@Autowired
	private OcConfig ocConfig;
	
	private HttpsUtil getHttpsClient() throws Exception {
		HttpsUtil httpsUtil = new HttpsUtil();
		
		String certPath = ocConfig.getSelfCertPath();
		String certPwd = ocConfig.getSelfCertPwd();
		
		String caPath = ocConfig.getTrustCaPath();
		String caPwd = ocConfig.getTrustCaPwd();
        httpsUtil.createSSLClient(certPath, certPwd, caPath, caPwd);
        return httpsUtil;
	}
	
	/**
	 * 订阅消息
	 * @param notifyType 消息类型
	 * @param callbackUrl 回调地址
	 * @return
	 * @throws Exception
	 */
	public OcRsp subscribe(String notifyType, String callbackUrl) throws Exception {
    	HttpsUtil httpsUtil = getHttpsClient();
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);
       
        String jsonRequest = JsonUtil.jsonObj2Sting(params);
        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, jsonRequest);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        logger.debug("device info Changed Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
    } 
    
    /**
     * 订阅设备添加通知
     * @throws Exception
     */
    public OcRsp deviceAdded(String callbackUrl) throws Exception {
    	HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
        if(StringUtils.isNotEmpty(callbackUrl) ) {
        	callbackUrl = ocConfig.getDeviceAddedCallbackUrl();
        }
        
        String notifyType = OcConfig.DEVICE_ADDED;
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);
        String jsonRequest = JsonUtil.jsonObj2Sting(params);
        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, jsonRequest);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        logger.debug("device info Changed Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
    }
    
    /**
     * 设备信息变化订阅通知
     * @throws Exception
     */
    public OcRsp deviceInfoChanged(String callbackUrl) throws Exception {
        
    	HttpsUtil httpsUtil = getHttpsClient();
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
        String notifyType = OcConfig.DEVICE_INFO_CHANGED;

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);

        String jsonRequest_deviceInfoChanged = JsonUtil.jsonObj2Sting(params);
        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, jsonRequest_deviceInfoChanged);
        String responseBody = httpsUtil.getHttpResponseBody(response);

        logger.debug("device info Changed Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
    }
    
    /**
     * 数据变化订阅通知
     * @throws Exception
     */
	public OcRsp deviceDataChanged(String callbackUrl) throws Exception {
		
		HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
		
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
        String notifyType = OcConfig.DEVICE_DATA_CHANGED;

        Map<String, Object> params = new HashMap<>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);

        String deviceDataChangedJson = JsonUtil.jsonObj2Sting(params);

        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, deviceDataChangedJson);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        
        logger.debug("device data Changed Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
	}
	
	/**
	 * 设备删除订阅通知
	 * @throws Exception
	 */
    public OcRsp deviceDeleted(String callbackUrl) throws Exception {
        
    	HttpsUtil httpsUtil = getHttpsClient();
        
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
        
        String notifyType = OcConfig.DEVICE_DELETED;
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);

        String deviceDeletedJson = JsonUtil.jsonObj2Sting(params);

        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, deviceDeletedJson);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        logger.debug("device data Changed Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
    }
    
    /**
     * 消息确认订阅通知
     * @throws Exception
     */
    public OcRsp messageConfirm(String callbackUrl) throws Exception {
        
    	HttpsUtil httpsUtil = getHttpsClient();
        
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
        String notifyType = OcConfig.MESSAGE_CONFIRM;

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);

        String messageConfirmJson = JsonUtil.jsonObj2Sting(params);

        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, messageConfirmJson);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        
        logger.debug("message confirm Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
    }
    
    /**
     * 服务信息变化订阅通知
     * @return
     * @throws Exception
     */
    public OcRsp serviceInfoChanged(String callbackUrl) throws Exception {
        
    	HttpsUtil httpsUtil = getHttpsClient();
        
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
       
        String notifyType = OcConfig.SERVICE_INFO_CHANGED;

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);

        String reqJsonPsrams = JsonUtil.jsonObj2Sting(params);

        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, reqJsonPsrams);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        logger.debug("service info changed Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
	}
    
    /**
     * 命令响应订阅通知
     * @throws Exception
     */
	public OcRsp commandResponse(String callbackUrl) throws Exception {
	        
		HttpsUtil httpsUtil = getHttpsClient();
        
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
      
        String notifyType = OcConfig.COMMAND_RSP;

        Map<String, Object> param = new HashMap<String, Object>();
        param.put("notifyType", notifyType);
        param.put("callbackUrl", callbackUrl);

        String reqJsonParams = JsonUtil.jsonObj2Sting(param);

        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, reqJsonParams);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        logger.debug("command response Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
	}
	
	/**
	 * 设备事件订阅通知
	 * @throws Exception
	 */
	public OcRsp deviceEvent(String callbackUrl) throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();
        
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
       
        String notifyType = OcConfig.DEVICE_EVENT;

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);

        String reqJosnParams = JsonUtil.jsonObj2Sting(params);

        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, reqJosnParams);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        logger.debug("command response Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
	}
	
	/**
	 * 规则事件订阅通知
	 * @throws Exception
	 */
	public OcRsp ruleEvent(String callbackUrl) throws Exception {
        
		HttpsUtil httpsUtil = getHttpsClient();
        
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String subscribeUrl = ocConfig.getSubscriptionsUrl();
        
        String notifyType = OcConfig.RULE_EVENT;
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("notifyType", notifyType);
        params.put("callbackUrl", callbackUrl);

        String reqJsonParams = JsonUtil.jsonObj2Sting(params);

        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        HttpResponse response = httpsUtil.doPostJson(subscribeUrl, header, reqJsonParams);
        String responseBody = httpsUtil.getHttpResponseBody(response);
        
        logger.debug("Rule event Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
	}

	/**
	 * 批量数据变化订阅通知
	 * @throws Exception
	 */
	public OcRsp deviceDatasChanged() throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();
        
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String urlSubscribe = ocConfig.getSubscriptionsUrl();
		String callbackUrl = ocConfig.getDeviceDatasChangedCallbackUrl();
		String notifyType = OcConfig.DEVICE_DATAS_CHANGED;
		
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("notifyType", notifyType);
		params.put("callbackUrl", callbackUrl);
		
		String reqJsonParams = JsonUtil.jsonObj2Sting(params);
		
		Map<String, String> header = new HashMap<>();
		header.put(OcConfig.HEADER_APP_KEY, appId);
		header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
		HttpResponse response = httpsUtil.doPostJson(urlSubscribe, header, reqJsonParams);
		String responseBody = httpsUtil.getHttpResponseBody(response);
		logger.debug("device datas changed Subscribe Notify, response content: status:{}, notifyType: {}, content:{}", 
        		response.getStatusLine().getStatusCode(), notifyType, responseBody);
		OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
    }

	/**
	 * 订阅查询
	 * @param notifyType
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	public OcRsp querySubscription(String notifyType, String pageNo, String pageSize) throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();
        
        String accessToken = tokenService.getToken().getAccessToken();
        String appId = ocConfig.getAppId();
        String urlSubscribe = ocConfig.getSubscriptionsUrl();
		//String notifyType = OcConfig.DEVICE_DATAS_CHANGED;
		
		Map<String, String> queryParams = new HashMap<String, String>();
		if(StringUtils.isNotEmpty(notifyType)) {
			queryParams.put("pageNo", notifyType);	
		}
		if(StringUtils.isNotEmpty(notifyType)) {
			queryParams.put("pageNo", pageNo);	
		}
		if(StringUtils.isNotEmpty(notifyType)) {
			queryParams.put("pageSize", pageSize);	
		}
		
		Map<String, String> header = new HashMap<String, String>();
		header.put(OcConfig.HEADER_APP_KEY, appId);
		header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
		HttpResponse response = httpsUtil.doGetWithParas(urlSubscribe, queryParams, header);
		String responseBody = httpsUtil.getHttpResponseBody(response);
		logger.debug("query Subscribe , response content: status:{}, content:{}", 
        		response.getStatusLine().getStatusCode(), responseBody);
		OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), responseBody);
        return rsp;
	}
	
}
