package com.hgzx.plp.ds.oc.service;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.ehcache.EhCacheCacheManager;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.hgzx.plp.ds.oc.common.utils.HttpsUtil;
import com.hgzx.plp.ds.oc.common.utils.JsonUtil;
import com.hgzx.plp.ds.oc.common.utils.StreamClosedHttpResponse;
import com.hgzx.plp.ds.oc.config.OcConfig;
import com.hgzx.plp.ds.oc.model.OcToken;

@Service
public class OcTokenService {
	
	private static final Logger logger = LoggerFactory.getLogger(OcTokenService.class);
	
	@Autowired
    private EhCacheCacheManager cacheManager;
	
	@Autowired
	private OcConfig ocConfig;
	
	private HttpsUtil getHttpsClient() throws Exception {
		HttpsUtil httpsUtil = new HttpsUtil();
		
		String certPath = ocConfig.getSelfCertPath();
		String certPwd = ocConfig.getSelfCertPwd();
		
		String caPath = ocConfig.getTrustCaPath();
		String caPwd = ocConfig.getTrustCaPwd();
		
        httpsUtil.createSSLClient(certPath, certPwd, caPath, caPwd);
        return httpsUtil;
	}
	
	/**
	 * 获取访问token
	 * @return
	 * @throws Exception
	 */
	public OcToken getToken() throws Exception {//OC_TOKEN
		OcToken token = cacheManager.getCache("OC_TOKEN").get("token", OcToken.class);
		if(token != null) {
			return token;
		}
		HttpsUtil httpsUtil = getHttpsClient();

        String appId = ocConfig.getAppId();
        String secret = ocConfig.getSecret();
        String loginUrl = ocConfig.getAppAuthUrl();
        Map<String, String> param = new HashMap<String, String>();
        param.put("appId", appId);
        param.put("secret", secret);
        StreamClosedHttpResponse response = httpsUtil.doPostFormUrlEncodedGetStatusLine(loginUrl, param);
        
        logger.debug("get access token, response content: status:{}, content:{}", response.getStatusLine().getStatusCode(), response.getContent());
        
        if(response != null && response.getStatusLine().getStatusCode() == 200) {
        	OcToken newToken = JSON.parseObject(response.getContent(), OcToken.class);
        	cacheManager.getCache("OC_TOKEN").put("token", newToken);
        	return newToken;
        } else {
        	throw new Exception(response.getContent());
        }
	}
	
	/**
	 * 刷新访问token
	 * @param refreshToken 刷新码
	 * @return
	 * @throws Exception
	 */
	public OcToken getRefreshToken(String refreshToken) throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();

        // get refreshToken
        String appId = ocConfig.getAppId();
        String secret = ocConfig.getSecret();
        String urlRefreshToken = ocConfig.getRefreshTokenUrl();

        Map<String, Object> param_reg = new HashMap<String, Object>();
        param_reg.put("appId", appId);
        param_reg.put("secret", secret);
        param_reg.put("refreshToken", refreshToken);

        String jsonRequest = JsonUtil.jsonObj2Sting(param_reg);
        StreamClosedHttpResponse response = httpsUtil.doPostJsonGetStatusLine(urlRefreshToken, jsonRequest);

        logger.debug("RefreshToken, response content: status:{}, content:{}", response.getStatusLine().getStatusCode(), response.getContent());
        
        if(response != null && response.getStatusLine().getStatusCode() == 200) {
        	OcToken newToken = JSON.parseObject(response.getContent(), OcToken.class);
        	cacheManager.getCache("OC_TOKEN").put("token", newToken);
        	return newToken;
        } else {
        	throw new Exception(response.getContent());
        }
    }
	
}
